package es.aranova.mhp.base.controls;
/************************************************************
 * The MapGet example application.
 * Class Keypad
 * - Implements a numeric keypad which is used for typing text
 *
 * Niklas Hagstr�m		
 * Turku Centre for Computer Science - http://www.tucs.fi/
 * September, 2004
 * 
 ************************************************************
 */

import java.awt.Color;
import java.awt.Container;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.KeyEvent;


public class Keypad extends Container implements Runnable {

	private class NewKey {
		private int nk = -1;
		public NewKey() { }
		public void set(int i) { nk = i; }
		public int get() { return nk; }
	}
	
	// Keypad size constants, don't change.
	private static final int width = 500;
	private static final int height = 237;

	private String editedText = null;
	private char pendingChar = 0;
	
	// Used for keypad output, set by setOutput method
	private EditableTextBox textBox = null;
	
	// Images of buttons
	private Image redButtonImage = null;
	private Image blueButtonImage = null;
	private Image okButtonImage = null;
	
	public Keypad() {
		setSize(width,height);
		// Load button images
		redButtonImage = Toolkit.getDefaultToolkit().getImage("images/button_red.png");
		blueButtonImage = Toolkit.getDefaultToolkit().getImage("images/button_blue.png");
		// Create numeric buttons and layout
		buttons = new KeypadButton[10];
		buttons[0] = new KeypadButton(0,"0");
		buttons[1] = new KeypadButton(1,"1");
		buttons[2] = new KeypadButton(2,"abc2äå");
		buttons[3] = new KeypadButton(3,"def3");
		buttons[4] = new KeypadButton(4,"ghi4");
		buttons[5] = new KeypadButton(5,"jkl5");
		buttons[6] = new KeypadButton(6,"mno6ö");
		buttons[7] = new KeypadButton(7,"pqrs7");
		buttons[8] = new KeypadButton(8,"tuv8");
		buttons[9] = new KeypadButton(9,"wxyz9");
		buttons[1].setLocation(259,5);
		buttons[2].setLocation(339,5);
		buttons[3].setLocation(419,5);
		buttons[4].setLocation(259,62);
		buttons[5].setLocation(339,62);
		buttons[6].setLocation(419,62);
		buttons[7].setLocation(259,119);
		buttons[8].setLocation(339,119);
		buttons[9].setLocation(419,119);
		buttons[0].setLocation(339,176);
		for (int i=0;i<10;i++) { 
			buttons[i].setVisible(true);
			add(buttons[i]);
		}
		newKey = new NewKey();
	}

	public void paint(Graphics g) {
		// Paint keypad border
		g.setColor(Color.black);
		g.fillRect(254,0,245,height);
		// Paint keypad background
		g.setColor(new Color(0xaa,0xaa,0xaa));
		g.fillRect(256,2,241,height-4);
		// Paint backspace button
		g.drawImage(redButtonImage,10,40,this);
		g.setColor(Color.black);
		g.setFont(new Font("Tireasias",Font.PLAIN,25));
		g.drawString("= backspace",90,71);		
		// Paint space button
		g.drawImage(blueButtonImage,10,90,this);
		g.drawString("= space",90,121);
		// Paint components (keys)
		super.paint(g);
	}
	
	// activeChar is the character in editText which is edited at the moment
	int activeChar = 0;
	
	private int numKeyActive = -1; // -1 = no key
	private char chosenChar = 0;   // 0 = no character
	private int timer = 0;
	boolean backSpaceSet = false;  // backspace has been pressed?
	boolean spaceSet = false;      // space has been pressed?
	
	private NewKey newKey = null;
	private KeypadButton[] buttons;
		
	// Main loop of the thread.
	public void run() {
		int i=0;
		while (true) {
			// Sleep for a while so that the CPU won't be too busy.
			try { Thread.sleep(100); } catch (Exception e) { System.out.println(e); }
			// newKey is shared by two thread, allow only one access at a time
			synchronized (newKey) {
				if (newKey.get() != -1) {
					if (numKeyActive == -1) {
						// New key pressed
						numKeyActive = newKey.get();
						buttons[numKeyActive].press();
						textBox.setPending(buttons[numKeyActive].getChar());
						newKey.set(-1);
					} else {
						if (numKeyActive == newKey.get()) {
							// Old key re-pressed.
							buttons[numKeyActive].press();
							textBox.setPending(buttons[numKeyActive].getChar());
							newKey.set(-1);
						} else {
							// New key pressed while old still active
							System.out.println("Chosen: "+buttons[numKeyActive].getChar());
							textBox.addPending();	
							buttons[numKeyActive].unpress();
							numKeyActive = newKey.get();
							buttons[numKeyActive].press();
							textBox.setPending(buttons[numKeyActive].getChar());
							newKey.set(-1);
						}
					}
					// Every key press sets the timer to 10
					timer = 10;
				} else {
					// If no key press, the timer is decremented. 
					timer--;
				}
				if (timer==0) {
					// If timer reaches 0 it means that it has gone 10x100ms=1 second since
					// the active key was pressed. The active key is now added to the edited
					// string (in the text box).
					System.out.println("Chosen: "+buttons[numKeyActive].getChar());
					textBox.addPending();
					buttons[numKeyActive].unpress();
					numKeyActive = -1;
				}
				// If a key is active, backspace always inactivates it.
				if (backSpaceSet) {
					textBox.backSpace();
					if (numKeyActive != -1) {
						buttons[numKeyActive].unpress();
						numKeyActive = -1;
					}
					timer = -1;
					backSpaceSet = false;
				}
				// A space is always added to the edited string. If a key is active
				// when space is pressed, it get inactivated.
				if (spaceSet) {
					textBox.space();
					if (numKeyActive != -1) {
						buttons[numKeyActive].unpress();
						numKeyActive = -1;
					}
					timer = -1;
					spaceSet = false;
				}
			
			}
		}
	}

	// Acknowledge key press.
	public void keyPressed (KeyEvent key) {   
		int code = key.getKeyCode();
		if ((code >= KeyEvent.VK_0) && (code <= KeyEvent.VK_9)) {
			// newKey can also be changed by the run method. 
			synchronized (newKey) {
				newKey.set(code-KeyEvent.VK_0);
			}
		}	
	}

	public void backSpace() {
		backSpaceSet = true;
	}
	
	public void space() {
		spaceSet = true;
	}
		
	// Assign the key pad to a EditableTextBox.
	public void setOutput (EditableTextBox e) {
		textBox = e;
	}
}